Module: Datadog::Tracing::Contrib::HTTPX::Plugin

Defined in:
lib/httpx/adapters/datadog.rb

Overview

HTTPX Datadog Plugin

Enables tracing for httpx requests.

A span will be created for each request transaction; the span is created lazily only when buffering a request, and it is fed the start time stored inside the tracer object.

Defined Under Namespace

Modules: RequestMethods, RequestTracer

Class Method Summary collapse

Class Method Details

.extra_options(options) ⇒ Object



176
177
178
# File 'lib/httpx/adapters/datadog.rb', line 176

def extra_options(options)
  options.merge(tracer: RequestTracer)
end

.load_dependencies(klass) ⇒ Object



172
173
174
# File 'lib/httpx/adapters/datadog.rb', line 172

def load_dependencies(klass)
  klass.plugin(:tracing)
end