Module: HTTPX::Plugins::FollowRedirects::OptionsMethods

Defined in:
lib/httpx/plugins/follow_redirects.rb

Overview

adds support for the following options:

:max_redirects :: max number of times a request will be redirected (defaults to 3). :follow_insecure_redirects :: whether redirects to an "http://" URI, when coming from an "https//", are allowed (defaults to false). :allow_auth_to_other_origins :: whether auth-related headers, such as "Authorization", are propagated on redirection (defaults to false). :redirect_on :: optional callback which receives the redirect location and can halt the redirect chain if it returns false.