Module: HTTPX::Plugins::Retries::OptionsMethods
- Defined in:
- lib/httpx/plugins/retries.rb
Overview
adds support for the following options:
:max_retries :: max number of times a request will be retried (defaults to 3). :retry_change_requests :: whether idempotent requests are retried (defaults to false). :retry_after:: seconds after which a request is retried; can also be a callable object (i.e. ->(req, res) { ... } ) or the name of a supported backoff algorithm (i.e. :exponential_backoff). :retry_jitter :: number of seconds applied to :retry_after (must be a callable, i.e. ->(retry_after) { ... } ). :retry_on :: callable which alternatively defines a different rule for when a response is to be retried (i.e. ->(res) { ... }).