Module: HTTPX::Plugins::Tracing
- Defined in:
- lib/httpx/plugins/tracing.rb,
sig/plugins/tracing.rbs
Overview
This plugin adds a simple interface to integrate request tracing SDKs.
An example of such an integration is the datadog adapter.
Defined Under Namespace
Modules: ConnectionMethods, OptionsMethods, RequestMethods, _RetriesOptions, _Tracer Classes: Wrapper