Class: HTTPX::Connection
Overview
The Connection can be watched for IO events.
It contains the io object to read/write from, and knows what to do when it can.
It defers connecting until absolutely necessary. Connection should be triggered from
the IO selector (until then, any request will be queued).
A connection boots up its parser after connection is established. All pending requests
will be redirected there after connection.
A connection can be prevented from closing by the parser, that is, if there are pending
requests. This will signal that the connection was prematurely closed, due to a possible
number of conditions:
- Remote peer closed the connection ("Connection: close");
- Remote peer doesn't support pipelining;
A connection may also route requests for a different host for which the io was connected
to, provided that the IP is the same and the port and scheme as well. This will allow to
share the same socket to send HTTP/2 requests to different hosts.
Defined Under Namespace
Classes: HTTP1, HTTP2
Constant Summary
Constants included
from Loggable
Loggable::COLORS
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from Callbacks
#callbacks_for?, #emit, #on, #once, #only
Methods included from Loggable
#log, #log_exception
Constructor Details
#initialize(uri, options) ⇒ Connection
Returns a new instance of Connection.
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
|
# File 'lib/httpx/connection.rb', line 50
def initialize(uri, options)
@origins = [uri.origin]
@origin = Utils.to_uri(uri.origin)
@options = Options.new(options)
@type = initialize_type(uri, @options)
@window_size = @options.window_size
@read_buffer = Buffer.new(@options.buffer_size)
@write_buffer = Buffer.new(@options.buffer_size)
@pending = []
on(:error, &method(:on_error))
if @options.io
transition(:already_open)
@io = build_socket
parser
else
transition(:idle)
end
@inflight = 0
@keep_alive_timeout = @options.timeout[:keep_alive_timeout]
@intervals = []
self.addresses = @options.addresses if @options.addresses
end
|
Instance Attribute Details
#family ⇒ Object
Returns the value of attribute family.
48
49
50
|
# File 'lib/httpx/connection.rb', line 48
def family
@family
end
|
#io ⇒ Object
Returns the value of attribute io.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def io
@io
end
|
#options ⇒ Object
Returns the value of attribute options.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def options
@options
end
|
#origin ⇒ Object
Returns the value of attribute origin.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def origin
@origin
end
|
#origins ⇒ Object
Returns the value of attribute origins.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def origins
@origins
end
|
#pending ⇒ Object
Returns the value of attribute pending.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def pending
@pending
end
|
#ssl_session ⇒ Object
Returns the value of attribute ssl_session.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def ssl_session
@ssl_session
end
|
#state ⇒ Object
Returns the value of attribute state.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def state
@state
end
|
#timers=(value) ⇒ Object
Sets the attribute timers
46
47
48
|
# File 'lib/httpx/connection.rb', line 46
def timers=(value)
@timers = value
end
|
#type ⇒ Object
Returns the value of attribute type.
44
45
46
|
# File 'lib/httpx/connection.rb', line 44
def type
@type
end
|
Class Method Details
.parser_type(protocol) ⇒ Object
723
724
725
726
727
728
729
730
|
# File 'lib/httpx/connection.rb', line 723
def parser_type(protocol)
case protocol
when "h2" then HTTP2
when "http/1.1" then HTTP1
else
raise Error, "unsupported protocol (##{protocol})"
end
end
|
Instance Method Details
#addresses ⇒ Object
88
89
90
|
# File 'lib/httpx/connection.rb', line 88
def addresses
@io && @io.addresses
end
|
#addresses=(addrs) ⇒ Object
this is a semi-private method, to be used by the resolver
to initiate the io object.
80
81
82
83
84
85
86
|
# File 'lib/httpx/connection.rb', line 80
def addresses=(addrs)
if @io
@io.add_addresses(addrs)
else
@io = build_socket(addrs)
end
end
|
#call ⇒ Object
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
|
# File 'lib/httpx/connection.rb', line 192
def call
case @state
when :idle
connect
consume
when :closed
return
when :closing
consume
transition(:closed)
when :open
consume
end
nil
end
|
#close ⇒ Object
208
209
210
211
212
|
# File 'lib/httpx/connection.rb', line 208
def close
transition(:active) if @state == :inactive
@parser.close if @parser
end
|
#coalescable?(connection) ⇒ Boolean
coalescable connections need to be mergeable!
but internally, #mergeable? is called before #coalescable?
124
125
126
127
128
129
130
131
132
133
|
# File 'lib/httpx/connection.rb', line 124
def coalescable?(connection)
if @io.protocol == "h2" &&
@origin.scheme == "https" &&
connection.origin.scheme == "https" &&
@io.can_verify_peer?
@io.verify_hostname(connection.origin.host)
else
@origin == connection.origin
end
end
|
#connecting? ⇒ Boolean
164
165
166
|
# File 'lib/httpx/connection.rb', line 164
def connecting?
@state == :idle
end
|
#create_idle(options = {}) ⇒ Object
135
136
137
|
# File 'lib/httpx/connection.rb', line 135
def create_idle(options = {})
self.class.new(@origin, @options.merge(options))
end
|
#deactivate ⇒ Object
274
275
276
|
# File 'lib/httpx/connection.rb', line 274
def deactivate
transition(:inactive)
end
|
#expired? ⇒ Boolean
105
106
107
108
109
|
# File 'lib/httpx/connection.rb', line 105
def expired?
return false unless @io
@io.expired?
end
|
#force_reset ⇒ Object
bypasses the state machine to force closing of connections still connecting.
only used for Happy Eyeballs v2.
222
223
224
225
|
# File 'lib/httpx/connection.rb', line 222
def force_reset
@state = :closing
transition(:closed)
end
|
#handle_socket_timeout(interval) ⇒ Object
282
283
284
285
286
287
288
289
290
291
292
293
294
|
# File 'lib/httpx/connection.rb', line 282
def handle_socket_timeout(interval)
@intervals.delete_if(&:elapsed?)
unless @intervals.empty?
return
end
error = HTTPX::TimeoutError.new(interval, "timed out while waiting on select")
error.set_backtrace(caller)
on_error(error)
end
|
#idling ⇒ Object
263
264
265
266
267
268
|
# File 'lib/httpx/connection.rb', line 263
def idling
purge_after_closed
@write_buffer.clear
transition(:idle)
@parser = nil if @parser
end
|
#inflight? ⇒ Boolean
168
169
170
|
# File 'lib/httpx/connection.rb', line 168
def inflight?
@parser && !@parser.empty? && !@write_buffer.empty?
end
|
#interests ⇒ Object
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
|
# File 'lib/httpx/connection.rb', line 172
def interests
if connecting?
connect
return @io.interests if connecting?
end
return :w unless @write_buffer.empty?
return @parser.interests if @parser
nil
end
|
#match?(uri, options) ⇒ Boolean
92
93
94
95
96
97
98
99
100
101
102
103
|
# File 'lib/httpx/connection.rb', line 92
def match?(uri, options)
return false if !used? && (@state == :closing || @state == :closed)
(
@origins.include?(uri.origin) &&
(@origins.size == 1 || @origin == uri.origin || (@io.is_a?(SSL) && @io.verify_hostname(uri.host)))
) && @options == options
end
|
#merge(connection) ⇒ Object
139
140
141
142
143
144
145
146
147
148
149
150
|
# File 'lib/httpx/connection.rb', line 139
def merge(connection)
@origins |= connection.instance_variable_get(:@origins)
if connection.ssl_session
@ssl_session = connection.ssl_session
@io.session_new_cb do |sess|
@ssl_session = sess
end if @io
end
connection.purge_pending do |req|
send(req)
end
end
|
#mergeable?(connection) ⇒ Boolean
111
112
113
114
115
116
117
118
119
120
|
# File 'lib/httpx/connection.rb', line 111
def mergeable?(connection)
return false if @state == :closing || @state == :closed || !@io
return false unless connection.addresses
(
(open? && @origin == connection.origin) ||
!(@io.addresses & (connection.addresses || [])).empty?
) && @options == connection.options
end
|
#open? ⇒ Boolean
278
279
280
|
# File 'lib/httpx/connection.rb', line 278
def open?
@state == :open || @state == :inactive
end
|
#purge_pending(&block) ⇒ Object
152
153
154
155
156
157
158
159
160
161
162
|
# File 'lib/httpx/connection.rb', line 152
def purge_pending(&block)
pendings = []
if @parser
@inflight -= @parser.pending.size
pendings << @parser.pending
end
pendings << @pending
pendings.each do |pending|
pending.reject!(&block)
end
end
|
#reset ⇒ Object
227
228
229
230
231
232
233
|
# File 'lib/httpx/connection.rb', line 227
def reset
return if @state == :closing || @state == :closed
transition(:closing)
transition(:closed)
end
|
#send(request) ⇒ Object
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
|
# File 'lib/httpx/connection.rb', line 235
def send(request)
if @parser && !@write_buffer.full?
if @response_received_at && @keep_alive_timeout &&
Utils.elapsed_time(@response_received_at) > @keep_alive_timeout
log(level: 3) { "keep alive timeout expired, pinging connection..." }
@pending << request
parser.ping
transition(:active) if @state == :inactive
return
end
send_request_to_parser(request)
else
@pending << request
end
end
|
#terminate ⇒ Object
214
215
216
217
218
|
# File 'lib/httpx/connection.rb', line 214
def terminate
@connected_at = nil if @state == :closed
close
end
|
#timeout ⇒ Object
255
256
257
258
259
260
261
|
# File 'lib/httpx/connection.rb', line 255
def timeout
return @timeout if @timeout
return @options.timeout[:connect_timeout] if @state == :idle
@options.timeout[:operation_timeout]
end
|
#to_io ⇒ Object
188
189
190
|
# File 'lib/httpx/connection.rb', line 188
def to_io
@io.to_io
end
|
#used? ⇒ Boolean
270
271
272
|
# File 'lib/httpx/connection.rb', line 270
def used?
@connected_at
end
|