Class: HTTPX::ErrorResponse

Inherits:
Object
  • Object
show all
Extended by:
Forwardable
Includes:
ErrorResponsePatternMatchExtensions, Loggable
Defined in:
lib/httpx/response.rb

Overview

Wraps an error which has happened while processing an HTTP Request. It has partial public API parity with HTTPX::Response, so users should rely on it to infer whether the returned response is one or the other.

response = HTTPX.get("https://some-domain/path") #=> response is HTTPX::Response or HTTPX::ErrorResponse
response.raise_for_status #=> raises if it wraps an error

Constant Summary

Constants included from Loggable

Loggable::COLORS, Loggable::USE_DEBUG_LOG

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from ErrorResponsePatternMatchExtensions

#deconstruct, #deconstruct_keys

Methods included from Loggable

#log, #log_exception, #log_redact, #log_redact_body, #log_redact_headers

Constructor Details

#initialize(request, error) ⇒ ErrorResponse

Returns a new instance of ErrorResponse.



288
289
290
291
292
293
294
295
# File 'lib/httpx/response.rb', line 288

def initialize(request, error)
  @request = request
  @response = request.response if request.response.is_a?(Response)
  @error = error
  @options = request.options
  log_exception(@error)
  finish!
end

Instance Attribute Details

#errorObject (readonly)

the wrapped exception.



280
281
282
# File 'lib/httpx/response.rb', line 280

def error
  @error
end

#requestObject (readonly)

the corresponding HTTPX::Request instance.



274
275
276
# File 'lib/httpx/response.rb', line 274

def request
  @request
end

#responseObject (readonly)

the HTTPX::Response instance, when there is one (i.e. error happens fetching the response).



277
278
279
# File 'lib/httpx/response.rb', line 277

def response
  @response
end

Instance Method Details

#<<(data) ⇒ Object

buffers lost chunks to error response



322
323
324
325
326
# File 'lib/httpx/response.rb', line 322

def <<(data)
  return unless @response

  @response << data
end

#closeObject

closes the error resources.



303
304
305
# File 'lib/httpx/response.rb', line 303

def close
  @response.close if @response
end

#finish!Object



312
313
314
# File 'lib/httpx/response.rb', line 312

def finish!
  @request.connection = nil
end

#finished?Boolean

always true for error responses.

Returns:

  • (Boolean)


308
309
310
# File 'lib/httpx/response.rb', line 308

def finished?
  true
end

#raise_for_statusObject

raises the wrapped exception.

Raises:



317
318
319
# File 'lib/httpx/response.rb', line 317

def raise_for_status
  raise @error
end

#to_sObject

returns the exception full message.



298
299
300
# File 'lib/httpx/response.rb', line 298

def to_s
  @error.full_message(highlight: false)
end